@@ -23,22 +23,21 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import logging
+from threading import local
 
 import six
 from django.contrib.auth.models import AbstractBaseUser
 from django.contrib.auth.models import AnonymousUser
 from django.contrib.auth.models import UserManager
 from django.core import validators
-from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.db.models.query import F
+from django.db import transaction
+from django.db.models.query import Q
 from django.db.utils import IntegrityError
 from django.utils.encoding import python_2_unicode_compatible
 from morango.models import Certificate
-from morango.models import MorangoMPTTModel
-from morango.models import MorangoMPTTTreeManager
 from morango.models import SyncableModel
 from morango.models import SyncableModelManager
 from mptt.models import TreeForeignKey
@@ -48,19 +47,20 @@ from .constants import facility_presets
 from .constants import morango_sync
 from .constants import role_kinds
 from .constants import user_kinds
+from .errors import IncompatibleDeviceSettingError
+from .errors import InvalidCollectionHierarchy
+from .errors import InvalidMembershipError
 from .errors import InvalidRoleKind
 from .errors import UserDoesNotHaveRoleError
-from .errors import UserHasRoleOnlyIndirectlyThroughHierarchyError
-from .errors import UserIsMemberOnlyIndirectlyThroughHierarchyError
 from .errors import UserIsNotFacilityUser
 from .errors import UserIsNotMemberError
-from .filters import HierarchyRelationsFilter
 from .permissions.auth import AllCanReadFacilityDataset
 from .permissions.auth import AnonUserCanReadFacilities
 from .permissions.auth import CoachesCanManageGroupsForTheirClasses
 from .permissions.auth import CoachesCanManageMembershipsForTheirGroups
 from .permissions.auth import CollectionSpecificRoleBasedPermissions
 from .permissions.auth import FacilityAdminCanEditForOwnFacilityDataset
+from .permissions.auth import MembersCanReadMembershipsOfTheirCollections
 from .permissions.base import BasePermissions
 from .permissions.base import RoleBasedPermissions
 from .permissions.general import IsAdminForOwnFacility
@@ -68,17 +68,50 @@ from .permissions.general import IsFromSameFacility
 from .permissions.general import IsOwn
 from .permissions.general import IsSelf
 from kolibri.core.auth.constants.demographics import choices as GENDER_CHOICES
+from kolibri.core.auth.constants.demographics import DEFERRED
 from kolibri.core.auth.constants.morango_sync import ScopeDefinitions
 from kolibri.core.device.utils import DeviceNotProvisioned
 from kolibri.core.device.utils import get_device_setting
 from kolibri.core.device.utils import set_device_settings
 from kolibri.core.errors import KolibriValidationError
 from kolibri.core.fields import DateTimeTzField
-from kolibri.core.utils.cache import NamespacedCacheProxy
 from kolibri.utils.time_utils import local_now
 
 logger = logging.getLogger(__name__)
-dataset_cache = NamespacedCacheProxy(cache, "dataset")
+
+
+class DatasetCache(local):
+    def __init__(self):
+        self.deactivate()
+
+    def __enter__(self):
+        self.activate()
+
+    def activate(self):
+        self._active = True
+
+    def __exit__(self, type, value, traceback):
+        self.deactivate()
+
+    def deactivate(self):
+        self._active = False
+        self.clear()
+
+    def clear(self):
+        self._cache = {}
+
+    def get(self, key):
+        if self._active:
+            return self._cache.get(key)
+        return None
+
+    def set(self, key, dataset_id):
+        if self._active:
+            self._cache[key] = dataset_id
+        return None
+
+
+dataset_cache = DatasetCache()
 
 
 def _has_permissions_class(obj):
@@ -134,8 +167,21 @@ class FacilityDataset(FacilityDataSyncableModel):
             return "FacilityDataset for {}".format(
                 Facility.objects.get(id=facilities[0].id)
             )
-        else:
-            return "FacilityDataset (no associated Facility)"
+        return "FacilityDataset (no associated Facility)"
+
+    def save(self, *args, **kwargs):
+        self.ensure_compatibility()
+        super(FacilityDataset, self).save(*args, **kwargs)
+
+    def ensure_compatibility(self, *args, **kwargs):
+        if self.learner_can_login_with_no_password and self.learner_can_edit_password:
+            raise IncompatibleDeviceSettingError(
+                "Device Settings [learner_can_login_with_no_password={}] & [learner_can_edit_password={}] "
+                "values incompatible together.".format(
+                    self.learner_can_login_with_no_password,
+                    self.learner_can_edit_password,
+                )
+            )
 
     def calculate_source_id(self):
         # if we don't already have a source ID, get one by generating a new root certificate, and using its ID
@@ -148,7 +194,8 @@ class FacilityDataset(FacilityDataSyncableModel):
     @staticmethod
     def compute_namespaced_id(partition_value, source_id_value, model_name):
         # assert partition_value.startswith(FacilityDataset.ID_PLACEHOLDER)
-        assert model_name == FacilityDataset.morango_model_name
+        if model_name != FacilityDataset.morango_model_name:
+            raise AssertionError
         # we use the source_id as the ID for the FacilityDataset
         return source_id_value
 
@@ -164,6 +211,15 @@ class FacilityDataset(FacilityDataSyncableModel):
             tree_id=self.get_root_certificate().tree_id
         ).exclude(_private_key=None)
 
+    def reset_to_default_settings(self, preset=None):
+        from kolibri.core.auth.constants.facility_presets import mappings
+
+        # use the current preset if it is not passed in
+        dataset_data = mappings[preset or self.preset]
+        for key, value in dataset_data.items():
+            setattr(self, key, value)
+        self.save()
+
 
 class AbstractFacilityDataModel(FacilityDataSyncableModel):
     """
@@ -176,6 +232,10 @@ class AbstractFacilityDataModel(FacilityDataSyncableModel):
     class Meta:
         abstract = True
 
+    @classmethod
+    def get_related_dataset_cache_key(cls, id, db_table):
+        return "{id}_{db_table}_dataset".format(id=id, db_table=db_table)
+
     def cached_related_dataset_lookup(self, related_obj_name):
         """
         Attempt to get the dataset_id either from the cache or the actual related obj instance.
@@ -184,8 +244,8 @@ class AbstractFacilityDataModel(FacilityDataSyncableModel):
         :return: the dataset_id associated with the related obj
         """
         field = self._meta.get_field(related_obj_name)
-        key = "{id}_{db_table}_dataset".format(
-            id=getattr(self, field.attname), db_table=field.related_model._meta.db_table
+        key = self.get_related_dataset_cache_key(
+            getattr(self, field.attname), field.related_model._meta.db_table
         )
         dataset_id = dataset_cache.get(key)
         if dataset_id is None:
@@ -193,7 +253,7 @@ class AbstractFacilityDataModel(FacilityDataSyncableModel):
                 dataset_id = getattr(self, related_obj_name).dataset_id
             except ObjectDoesNotExist as e:
                 raise ValidationError(e)
-            dataset_cache.set(key, dataset_id, 60 * 10)
+            dataset_cache.set(key, dataset_id)
         return dataset_id
 
     def calculate_source_id(self):
@@ -211,15 +271,17 @@ class AbstractFacilityDataModel(FacilityDataSyncableModel):
         )
         super(AbstractFacilityDataModel, self).full_clean(*args, **kwargs)
 
-    def save(self, *args, **kwargs):
-
-        # before saving, ensure we have a dataset, and convert any validation errors into integrity errors,
-        # since by this point the `clean_fields` method should already have prevented this situation from arising
+    def pre_save(self):
+        # before saving, ensure we have a dataset, and convert any validation errors into integrity
+        # errors, since by this point the `clean_fields` method should already have prevented
+        # this situation from arising
         try:
             self.ensure_dataset()
         except KolibriValidationError as e:
             raise IntegrityError(str(e))
 
+    def save(self, *args, **kwargs):
+        self.pre_save()
         super(AbstractFacilityDataModel, self).save(*args, **kwargs)
 
     def ensure_dataset(self, *args, **kwargs):
@@ -313,30 +375,6 @@ class KolibriAbstractBaseUser(AbstractBaseUser):
             "Subclasses of KolibriAbstractBaseUser must override the `is_member_of` method."
         )
 
-    def get_roles_for_user(self, user):
-        """
-        Determine all the roles this user has in relation to the target user, and return a set containing the kinds of roles.
-
-        :param user: The target user for which this user has the roles.
-        :return: The kinds of roles this user has with respect to the target user.
-        :rtype: set of ``kolibri.core.auth.constants.role_kinds.*`` strings
-        """
-        raise NotImplementedError(
-            "Subclasses of KolibriAbstractBaseUser must override the `get_roles_for_user` method."
-        )
-
-    def get_roles_for_collection(self, coll):
-        """
-        Determine all the roles this user has in relation to the specified ``Collection``, and return a set containing the kinds of roles.
-
-        :param coll: The target ``Collection`` for which this user has the roles.
-        :return: The kinds of roles this user has with respect to the specified ``Collection``.
-        :rtype: set of ``kolibri.core.auth.constants.role_kinds.*`` strings
-        """
-        raise NotImplementedError(
-            "Subclasses of KolibriAbstractBaseUser must override the `get_roles_for_collection` method."
-        )
-
     def has_role_for_user(self, kinds, user):
         """
         Determine whether this user has (at least one of) the specified role kind(s) in relation to the specified user.
@@ -455,19 +493,6 @@ class KolibriAbstractBaseUser(AbstractBaseUser):
             "Subclasses of KolibriAbstractBaseUser must override the `can_delete` method."
         )
 
-    def get_roles_for(self, obj):
-        """
-        Helper function that defers to ``get_roles_for_user`` or ``get_roles_for_collection`` based on the type of object passed in.
-        """
-        if isinstance(obj, KolibriAbstractBaseUser):
-            return self.get_roles_for_user(obj)
-        elif isinstance(obj, Collection):
-            return self.get_roles_for_collection(obj)
-        else:
-            raise ValueError(
-                "The `obj` argument to `get_roles_for` must be either an instance of KolibriAbstractBaseUser or Collection."
-            )
-
     def has_role_for(self, kinds, obj):
         """
         Helper function that defers to ``has_role_for_user`` or ``has_role_for_collection`` based on the type of object passed in.
@@ -514,12 +539,6 @@ class KolibriAnonymousUser(AnonymousUser, KolibriAbstractBaseUser):
     def is_member_of(self, coll):
         return False
 
-    def get_roles_for_user(self, user):
-        return set([])
-
-    def get_roles_for_collection(self, coll):
-        return set([])
-
     def has_role_for_user(self, kinds, user):
         return False
 
@@ -530,38 +549,33 @@ class KolibriAnonymousUser(AnonymousUser, KolibriAbstractBaseUser):
         # check the object permissions, if available, just in case permissions are granted to anon users
         if _has_permissions_class(obj):
             return obj.permissions.user_can_create_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_read(self, obj):
         # check the object permissions, if available, just in case permissions are granted to anon users
         if _has_permissions_class(obj):
             return obj.permissions.user_can_read_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_update(self, obj):
         # check the object permissions, if available, just in case permissions are granted to anon users
         if _has_permissions_class(obj):
             return obj.permissions.user_can_update_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_delete(self, obj):
         # check the object permissions, if available, just in case permissions are granted to anon users
         if _has_permissions_class(obj):
             return obj.permissions.user_can_delete_object(self, obj)
-        else:
-            return False
+        return False
 
     def filter_readable(self, queryset):
         # check the object permissions, if available, just in case permissions are granted to anon users
         if _has_permissions_class(queryset.model):
-            return queryset.model.permissions.readable_by_user_filter(
-                self, queryset
+            return queryset.filter(
+                queryset.model.permissions.readable_by_user_filter(self)
             ).distinct()
-        else:
-            return queryset.none()
+        return queryset.none()
 
 
 class FacilityUserModelManager(SyncableModelManager, UserManager):
@@ -583,7 +597,7 @@ class FacilityUserModelManager(SyncableModelManager, UserManager):
         user.save(using=self._db)
         return user
 
-    def create_superuser(self, username, password, facility=None):
+    def create_superuser(self, username, password, facility=None, full_name=None):
 
         # import here to avoid circularity
         from kolibri.core.device.models import DevicePermissions
@@ -596,8 +610,15 @@ class FacilityUserModelManager(SyncableModelManager, UserManager):
             raise ValidationError("An account with that username already exists")
 
         # create the new account in that facility
+        # gender and birth_year are set to DEFERRED, since superusers do not
+        # need to provide this and are not nudged to update profile on Learn page
         superuser = FacilityUser(
-            full_name=username, username=username, password=password, facility=facility
+            full_name=full_name or username,
+            username=username,
+            password=password,
+            facility=facility,
+            gender=DEFERRED,
+            birth_year=DEFERRED,
         )
         superuser.full_clean()
         superuser.set_password(password)
@@ -610,6 +631,7 @@ class FacilityUserModelManager(SyncableModelManager, UserManager):
         DevicePermissions.objects.create(
             user=superuser, is_superuser=True, can_manage_content=True
         )
+        return superuser
 
 
 def validate_birth_year(value):
@@ -640,6 +662,24 @@ def validate_birth_year(value):
         raise ValidationError(error)
 
 
+role_kinds_set = {r[0] for r in role_kinds.choices}
+
+
+def validate_role_kinds(kinds):
+    if isinstance(kinds, six.string_types):
+        kinds = set([kinds])
+    else:
+        try:
+            kinds = set(kinds)
+        except TypeError:
+            raise TypeError(
+                "kinds argument must be a string or an iterable coerceable to a set"
+            )
+    if not role_kinds_set.issuperset(kinds):
+        raise InvalidRoleKind("kinds argument must only contain valid role kind names")
+    return kinds
+
+
 @python_2_unicode_compatible
 class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
     """
@@ -661,6 +701,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         can_be_read_by=(role_kinds.ADMIN, role_kinds.COACH),
         can_be_updated_by=(role_kinds.ADMIN,),
         can_be_deleted_by=(),  # don't want a classroom admin deleting a user completely, just removing them from the class
+        collection_field="memberships__collection",
     )
     permissions = own | admin | role
 
@@ -722,7 +763,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
             target_user = FacilityUser.objects.get(id=scope_params.get("user_id"))
             if self == target_user:
                 return True
-            if self.has_role_for_user(target_user, role_kinds.ADMIN):
+            if self.has_role_for_user(role_kinds.ADMIN, target_user):
                 return True
             return False
         return False
@@ -762,95 +803,48 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         if self.dataset_id != coll.dataset_id:
             return False
         if coll.kind == collection_kinds.FACILITY:
-            return True  # FacilityUser is always a member of her own facility
-        return (
-            HierarchyRelationsFilter(FacilityUser.objects.all())
-            .filter_by_hierarchy(target_user=F("id"), ancestor_collection=coll.id)
-            .filter(id=self.id)
-            .exists()
-        )
-
-    def get_roles_for_user(self, user):
-        if self.is_superuser:
-            # a superuser has admin role for all users on the device
-            return set([role_kinds.ADMIN])
-        if not hasattr(user, "dataset_id") or self.dataset_id != user.dataset_id:
-            return set([])
-        role_instances = (
-            HierarchyRelationsFilter(Role)
-            .filter_by_hierarchy(
-                ancestor_collection=F("collection"),
-                source_user=F("user"),
-                target_user=user,
-            )
-            .filter(user=self)
-        )
-        return set(
-            [instance["kind"] for instance in role_instances.values("kind").distinct()]
-        )
-
-    def get_roles_for_collection(self, coll):
-        if self.is_superuser:
-            # a superuser has admin role for all collections on the device
-            return set([role_kinds.ADMIN])
-        if self.dataset_id != coll.dataset_id:
-            return set([])
-        role_instances = (
-            HierarchyRelationsFilter(Role)
-            .filter_by_hierarchy(
-                ancestor_collection=F("collection"),
-                source_user=F("user"),
-                descendant_collection=coll,
-            )
-            .filter(user=self)
-        )
-        return set(
-            [instance["kind"] for instance in role_instances.values("kind").distinct()]
-        )
+            return self.facility_id == coll.id
+        return Membership.objects.filter(user=self, collection=coll).exists()
 
     def has_role_for_user(self, kinds, user):
+        kinds = validate_role_kinds(kinds)
         if self.is_superuser:
-            if isinstance(kinds, six.string_types):
-                kinds = [kinds]
             # a superuser has admin role for all users on the device
             return role_kinds.ADMIN in kinds
         if not kinds:
             return False
         if not hasattr(user, "dataset_id") or self.dataset_id != user.dataset_id:
             return False
-        return (
-            HierarchyRelationsFilter(Role)
-            .filter_by_hierarchy(
-                ancestor_collection=F("collection"),
-                source_user=F("user"),
-                role_kind=kinds,
-                target_user=user,
+        return Role.objects.filter(
+            Q(user=self, collection_id=user.facility_id, kind__in=kinds)
+            | Q(
+                user=self,
+                collection_id__in=user.memberships.all().values_list(
+                    "collection_id", flat=True
+                ),
+                kind__in=kinds,
             )
-            .filter(user=self)
-            .exists()
-        )
+        ).exists()
 
     def has_role_for_collection(self, kinds, coll):
+        kinds = validate_role_kinds(kinds)
         if self.is_superuser:
-            if isinstance(kinds, six.string_types):
-                kinds = [kinds]
             # a superuser has admin role for all collections on the device
             return role_kinds.ADMIN in kinds
         if not kinds:
             return False
         if self.dataset_id != coll.dataset_id:
             return False
-        return (
-            HierarchyRelationsFilter(Role)
-            .filter_by_hierarchy(
-                ancestor_collection=F("collection"),
-                source_user=F("user"),
-                role_kind=kinds,
-                descendant_collection=coll,
-            )
-            .filter(user=self)
-            .exists()
-        )
+        coll_id = coll.id
+        if (
+            coll.kind == collection_kinds.LEARNERGROUP
+            or coll.kind == collection_kinds.ADHOCLEARNERSGROUP
+        ):
+            coll_id = coll.parent_id
+        return Role.objects.filter(
+            Q(user=self, collection_id=self.facility_id, kind__in=kinds)
+            | Q(user=self, collection_id=coll_id, kind__in=kinds)
+        ).exists()
 
     def can_create_instance(self, obj):
         if self.is_superuser:
@@ -858,8 +852,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         # a FacilityUser's permissions are determined through the object's permission class
         if _has_permissions_class(obj):
             return obj.permissions.user_can_create_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_read(self, obj):
         if self.is_superuser:
@@ -867,8 +860,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         # a FacilityUser's permissions are determined through the object's permission class
         if _has_permissions_class(obj):
             return obj.permissions.user_can_read_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_update(self, obj):
         # Superusers cannot update their own permissions, because they only thing they can do is make themselves
@@ -878,8 +870,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         # a FacilityUser's permissions are determined through the object's permission class
         if _has_permissions_class(obj):
             return obj.permissions.user_can_update_object(self, obj)
-        else:
-            return False
+        return False
 
     def can_delete(self, obj):
         # Users cannot delete themselves
@@ -892,18 +883,16 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
         # a FacilityUser's permissions are determined through the object's permission class
         if _has_permissions_class(obj):
             return obj.permissions.user_can_delete_object(self, obj)
-        else:
-            return False
+        return False
 
     def filter_readable(self, queryset):
         if self.is_superuser:
             return queryset
         if _has_permissions_class(queryset.model):
-            return queryset.model.permissions.readable_by_user_filter(
-                self, queryset
+            return queryset.filter(
+                queryset.model.permissions.readable_by_user_filter(self)
             ).distinct()
-        else:
-            return queryset.none()
+        return queryset.none()
 
     def __str__(self):
         return '"{user}"@"{facility}"'.format(
@@ -927,7 +916,7 @@ class FacilityUser(KolibriAbstractBaseUser, AbstractFacilityDataModel):
 
 
 @python_2_unicode_compatible
-class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
+class Collection(AbstractFacilityDataModel):
     """
     ``Collections`` are hierarchical groups of ``FacilityUsers``, used for grouping users and making decisions about permissions.
     ``FacilityUsers`` can have roles for one or more ``Collections``, by way of obtaining ``Roles`` associated with those ``Collections``.
@@ -951,7 +940,7 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
     _KIND = None  # Should be overridden in subclasses to specify what "kind" they are
 
     name = models.CharField(max_length=100)
-    parent = TreeForeignKey(
+    parent = models.ForeignKey(
         "self", null=True, blank=True, related_name="children", db_index=True
     )
     kind = models.CharField(max_length=20, choices=collection_kinds.choices)
@@ -984,24 +973,22 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
             return FacilityUser.objects.filter(
                 dataset=self.dataset
             )  # FacilityUser is always a member of her own facility
-        return HierarchyRelationsFilter(FacilityUser).filter_by_hierarchy(
-            target_user=F("id"), ancestor_collection=self
-        )
+        return FacilityUser.objects.filter(memberships__collection=self)
 
     def get_coaches(self):
         """
         Returns users who have the coach role for this immediate collection.
         """
-        return HierarchyRelationsFilter(FacilityUser).filter_by_hierarchy(
-            source_user=F("id"), ancestor_collection=self, role_kind=role_kinds.COACH
+        return FacilityUser.objects.filter(dataset_id=self.dataset_id).filter(
+            roles__kind=role_kinds.COACH, roles__collection=self
         )
 
     def get_admins(self):
         """
         Returns users who have the admin role for this immediate collection.
         """
-        return HierarchyRelationsFilter(FacilityUser).filter_by_hierarchy(
-            source_user=F("id"), ancestor_collection=self, role_kind=role_kinds.ADMIN
+        return FacilityUser.objects.filter(dataset_id=self.dataset_id).filter(
+            roles__kind=role_kinds.ADMIN, roles__collection=self
         )
 
     def add_role(self, user, role_kind):
@@ -1015,7 +1002,7 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
         """
 
         # ensure the specified role kind is valid
-        if role_kind not in (kind[0] for kind in role_kinds.choices):
+        if role_kind not in role_kinds_set:
             raise InvalidRoleKind(
                 "'{role_kind}' is not a valid role kind.".format(role_kind=role_kind)
             )
@@ -1040,7 +1027,7 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
         """
 
         # ensure the specified role kind is valid
-        if role_kind not in (kind[0] for kind in role_kinds.choices):
+        if role_kind not in role_kinds_set:
             raise InvalidRoleKind(
                 "'{role_kind}' is not a valid role kind.".format(role_kind=role_kind)
             )
@@ -1050,21 +1037,15 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
             raise UserIsNotFacilityUser("You can only remove roles for FacilityUsers.")
 
         # make sure the user has the role to begin with
-        if not user.has_role_for_collection(role_kind, self):
+        try:
+            role = Role.objects.get(user=user, collection=self, kind=role_kind)
+        except Role.DoesNotExist:
             raise UserDoesNotHaveRoleError(
                 "User does not have this role for this collection."
             )
 
         # delete the appropriate role, if it exists
-        results = Role.objects.filter(
-            user=user, collection=self, kind=role_kind
-        ).delete()
-
-        # if no Roles were deleted, the user's role must have been indirect (via the collection hierarchy)
-        if results[0] == 0:
-            raise UserHasRoleOnlyIndirectlyThroughHierarchyError(
-                "Role cannot be removed, as user has it only indirectly, through the collection hierarchy."
-            )
+        role.delete()
 
     def add_member(self, user):
         """
@@ -1107,23 +1088,18 @@ class Collection(MorangoMPTTModel, AbstractFacilityDataModel):
                 "The user is not a member of the collection, and cannot be removed."
             )
 
-        # delete the appropriate membership, if it exists
-        results = Membership.objects.filter(user=user, collection=self).delete()
-
-        # if no Memberships were deleted, the user's membership must have been indirect (via the collection hierarchy)
-        if results[0] == 0:
-            raise UserIsMemberOnlyIndirectlyThroughHierarchyError(
-                "Membership cannot be removed, as user is a member only indirectly, through the collection hierarchy."
-            )
+        # delete the appropriate membership
+        Membership.objects.filter(user=user, collection=self).delete()
 
     def infer_dataset(self, *args, **kwargs):
+        if self.dataset_id:
+            return self.dataset_id
         if self.parent:
             # subcollections inherit dataset from root of their tree
             # (we can't call `get_root` directly on self, as it won't work if self hasn't yet been saved)
-            return self.parent.get_root().dataset_id
-        else:
-            # the root node (i.e. Facility) must be explicitly tied to a dataset
-            return None
+            return self.parent.dataset_id
+        # the root node (i.e. Facility) must be explicitly tied to a dataset
+        return None
 
     def __str__(self):
         return '"{name}" ({kind})'.format(name=self.name, kind=self.kind)
@@ -1153,7 +1129,9 @@ class Membership(AbstractFacilityDataModel):
     )
     # Membership can be written by coaches under the coaches' group
     membership = CoachesCanManageMembershipsForTheirGroups()
-    permissions = own | role | membership
+    # Members can read memberships of collections they are members of
+    own_collections = MembersCanReadMembershipsOfTheirCollections()
+    permissions = own | role | membership | own_collections
 
     user = models.ForeignKey(
         "FacilityUser", related_name="memberships", blank=False, null=False
@@ -1187,6 +1165,37 @@ class Membership(AbstractFacilityDataModel):
             user=self.user, collection=self.collection
         )
 
+    def save(self, *args, **kwargs):
+        if self.collection.kind == collection_kinds.FACILITY:
+            raise InvalidMembershipError(
+                "Cannot create membership objects for facilities, as should already be a member by facility attribute"
+            )
+        # Can skip checking that the user facility and the classroom are properly related,
+        # as infer dataset will check they have the same dataset
+        if (
+            self.collection.kind == collection_kinds.LEARNERGROUP
+            or self.collection.kind == collection_kinds.ADHOCLEARNERSGROUP
+        ):
+            if not Membership.objects.filter(
+                collection_id=self.collection.parent_id, user=self.user
+            ).exists():
+                raise InvalidMembershipError(
+                    "Cannot create membership for a user in a LearnerGroup or AdHoGroup when they are not a member of the parent Classrooom"
+                )
+        return super(Membership, self).save(*args, **kwargs)
+
+    def delete(self, **kwargs):
+        with transaction.atomic():
+            # Wrap in a transaction so we don't accidentally wipe out child memberships
+            # when deleting the parent membership fails
+            if self.collection.kind == collection_kinds.CLASSROOM:
+                # If membership is membership of classroom, should also cleanup all
+                # memberships of child entities - Learner Groups and AdHoc Groups.
+                Membership.objects.filter(
+                    user=self.user, collection__in=self.collection.children.all()
+                ).delete()
+            return super(Membership, self).delete(**kwargs)
+
 
 @python_2_unicode_compatible
 class Role(AbstractFacilityDataModel):
@@ -1250,8 +1259,50 @@ class Role(AbstractFacilityDataModel):
             user=self.user, kind=self.kind, collection=self.collection
         )
 
+    def save(self, *args, **kwargs):
+        if (
+            self.collection.kind == collection_kinds.LEARNERGROUP
+            or self.collection.kind == collection_kinds.ADHOCLEARNERSGROUP
+        ):
+            # We do not currently support roles at the learner group or ad hoc group level
+            raise InvalidRoleKind(
+                "Cannot assign roles to Learner Groups or AdHoc Groups"
+            )
+        with transaction.atomic():
+            if self.collection.kind == collection_kinds.CLASSROOM:
+                # We only support coaches to be assigned at the classroom level currently
+                if self.kind != role_kinds.COACH:
+                    raise InvalidRoleKind("Can only assign Coach roles to Classrooms")
+                if not Role.objects.filter(
+                    user=self.user, collection_id=self.collection.parent_id
+                ).exists():
+                    # If the user doesn't already have a facility role, then create the assignable coach role for the user
+                    # at the facility level.
+                    Role.objects.create(
+                        user=self.user,
+                        collection_id=self.collection.parent_id,
+                        kind=role_kinds.ASSIGNABLE_COACH,
+                    )
+            return super(Role, self).save(*args, **kwargs)
 
-class CollectionProxyManager(MorangoMPTTTreeManager):
+    def delete(self, **kwargs):
+        with transaction.atomic():
+            # Wrap in a transaction so we don't accidentally wipe out role assignments
+            # when the base delete fails.
+            if (
+                self.collection.kind == collection_kinds.FACILITY
+                and self.kind == role_kinds.ASSIGNABLE_COACH
+            ):
+                # If deleting the ASSIGNABLE_COACH role, also delete any classroom coach roles
+                Role.objects.filter(
+                    user=self.user,
+                    collection__in=self.collection.children.all(),
+                    kind=role_kinds.COACH,
+                ).delete()
+            return super(Role, self).delete(**kwargs)
+
+
+class CollectionProxyManager(SyncableModelManager):
     def get_queryset(self):
         return (
             super(CollectionProxyManager, self)
@@ -1357,6 +1408,12 @@ class Classroom(Collection):
             raise IntegrityError(
                 "Classroom cannot be the root of a collection tree, and must have a parent."
             )
+
+        if not self.parent.kind == collection_kinds.FACILITY:
+            raise InvalidCollectionHierarchy(
+                "Classroom must be the child of a Facility"
+            )
+
         super(Classroom, self).save(*args, **kwargs)
 
     def get_facility(self):
@@ -1422,6 +1479,10 @@ class LearnerGroup(Collection):
             raise IntegrityError(
                 "LearnerGroup cannot be the root of a collection tree, and must have a parent."
             )
+        if not self.parent.kind == collection_kinds.CLASSROOM:
+            raise InvalidCollectionHierarchy(
+                "LearnerGroup must be the child of a Classroom"
+            )
         super(LearnerGroup, self).save(*args, **kwargs)
 
     def get_classroom(self):
@@ -1461,11 +1522,24 @@ class AdHocGroup(Collection):
     class Meta:
         proxy = True
 
+    @classmethod
+    def deserialize(cls, dict_model):
+        # be defensive against blank names, set to `Ad hoc` if blank
+        name = dict_model.get("name", "") or ""
+        if len(name) == 0:
+            dict_model.update(name="Ad hoc")
+
+        return super(AdHocGroup, cls).deserialize(dict_model)
+
     def save(self, *args, **kwargs):
         if not self.parent:
             raise IntegrityError(
                 "AdHocGroup cannot be the root of a collection tree, and must have a parent."
             )
+        if not self.parent.kind == collection_kinds.CLASSROOM:
+            raise InvalidCollectionHierarchy(
+                "AdHocGroup must be the child of a Classroom"
+            )
         super(AdHocGroup, self).save(*args, **kwargs)
 
     def get_classroom(self):
