@@ -1,7 +1,7 @@
+import concurrent.futures
 import logging
 import os
 
-import concurrent.futures
 import requests
 from django.core.management.base import CommandError
 from le_utils.constants import content_kinds
@@ -9,17 +9,22 @@ from le_utils.constants import content_kinds
 from ...utils import annotation
 from ...utils import paths
 from ...utils import transfer
+from kolibri.core.content.errors import InsufficientStorageSpaceError
 from kolibri.core.content.errors import InvalidStorageFilenameError
+from kolibri.core.content.models import ChannelMetadata
 from kolibri.core.content.models import ContentNode
 from kolibri.core.content.utils.file_availability import LocationError
 from kolibri.core.content.utils.import_export_content import compare_checksums
 from kolibri.core.content.utils.import_export_content import get_import_export_data
 from kolibri.core.content.utils.paths import get_channel_lookup_url
+from kolibri.core.content.utils.paths import get_content_file_name
 from kolibri.core.content.utils.upgrade import get_import_data_for_update
 from kolibri.core.tasks.management.commands.base import AsyncCommand
-from kolibri.core.tasks.utils import db_task_write_lock
 from kolibri.core.tasks.utils import get_current_job
 from kolibri.utils import conf
+from kolibri.utils.options import FD_PER_THREAD
+from kolibri.utils.system import get_fd_limit
+from kolibri.utils.system import get_free_space
 
 # constants to specify the transfer method to be used
 DOWNLOAD_METHOD = "download"
@@ -245,6 +250,14 @@ class Command(AsyncCommand):
                 )
             raise
 
+        if not paths.using_remote_storage():
+            free_space = get_free_space(conf.OPTIONS["Paths"]["CONTENT_DIR"])
+
+            if free_space <= total_bytes_to_transfer:
+                raise InsufficientStorageSpaceError(
+                    "Import would completely fill remaining disk space"
+                )
+
         job = get_current_job()
 
         if job:
@@ -275,68 +288,110 @@ class Command(AsyncCommand):
             node_ids, exclude_node_ids, total_bytes_to_transfer
         )
 
-        with self.start_progress(
-            total=total_bytes_to_transfer + dummy_bytes_for_annotation
-        ) as overall_progress_update:
+        if paths.using_remote_storage():
+            overall_progress_update = self.start_progress(
+                total=dummy_bytes_for_annotation
+            ).update_progress
+            file_checksums_to_annotate.extend(f["id"] for f in files_to_download)
+            transferred_file_size = total_bytes_to_transfer
+        else:
+            remaining_bytes_to_transfer = total_bytes_to_transfer
+            overall_progress_update = self.start_progress(
+                total=total_bytes_to_transfer + dummy_bytes_for_annotation
+            ).update_progress
             if method == DOWNLOAD_METHOD:
                 session = requests.Session()
 
-            file_transfers = []
-            for f in files_to_download:
+            executor = (
+                concurrent.futures.ProcessPoolExecutor
+                if conf.OPTIONS["Tasks"]["USE_WORKER_MULTIPROCESSING"]
+                else concurrent.futures.ThreadPoolExecutor
+            )
 
-                if self.is_cancelled():
-                    break
+            max_workers = 10
 
-                filename = f.get_filename()
-                try:
-                    dest = paths.get_content_storage_file_path(filename)
-                except InvalidStorageFilenameError:
-                    # If the destination file name is malformed, just stop now.
-                    overall_progress_update(f.file_size)
-                    continue
+            if not conf.OPTIONS["Tasks"]["USE_WORKER_MULTIPROCESSING"]:
+                # If we're not using multiprocessing for workers, we may need
+                # to limit the number of workers depending on the number of allowed
+                # file descriptors.
+                # This is a heuristic method, where we know there can be issues if
+                # the max number of file descriptors for a process is 256, and we use 10
+                # workers, with potentially 4 concurrent tasks downloading files.
+                # The number of concurrent tasks that might be downloading files is determined
+                # by the number of regular workers running in the task runner
+                # (although the high priority task queue could also be running a channel database download).
+                server_reserved_fd_count = (
+                    FD_PER_THREAD * conf.OPTIONS["Server"]["CHERRYPY_THREAD_POOL"]
+                )
+                max_descriptors_per_download_task = (
+                    get_fd_limit() - server_reserved_fd_count
+                ) / conf.OPTIONS["Tasks"]["REGULAR_PRIORITY_WORKERS"]
+                # Each download task only needs to have a maximum of two open file descriptors at once:
+                # The temporary download file that the file is streamed to initially, and then
+                # the actual destination file that it is moved to. To add tolerance, we divide
+                # the number of file descriptors that could be allocated to this task by four,
+                # which should give us leeway in case of unforeseen descriptor use during the process.
+                max_workers = min(
+                    max_workers, min(1, max_descriptors_per_download_task // 4)
+                )
 
-                # if the file already exists, add its size to our overall progress, and skip
-                if os.path.isfile(dest) and os.path.getsize(dest) == f.file_size:
-                    overall_progress_update(f.file_size)
-                    file_checksums_to_annotate.append(f.id)
-                    transferred_file_size += f.file_size
-                    continue
-
-                # determine where we're downloading/copying from, and create appropriate transfer object
-                if method == DOWNLOAD_METHOD:
-                    url = paths.get_content_storage_remote_url(
-                        filename, baseurl=baseurl
-                    )
-                    filetransfer = transfer.FileDownload(
-                        url, dest, session=session, cancel_check=self.is_cancelled
-                    )
-                    file_transfers.append((f, filetransfer))
-                elif method == COPY_METHOD:
-                    try:
-                        srcpath = paths.get_content_storage_file_path(
-                            filename, datafolder=path
-                        )
-                    except InvalidStorageFilenameError:
-                        # If the source file name is malformed, just stop now.
-                        overall_progress_update(f.file_size)
-                        continue
-                    filetransfer = transfer.FileCopy(
-                        srcpath, dest, cancel_check=self.is_cancelled
-                    )
-                    file_transfers.append((f, filetransfer))
-
-            with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
+            with executor(max_workers=max_workers) as executor:
                 batch_size = 100
                 # ThreadPoolExecutor allows us to download files concurrently,
                 # greatly reducing download time in most cases. However, loading
                 # all the downloads into the pool requires considerable memory,
                 # so we divide the downloads into batches to keep memory usage down.
                 # In batches of 100, total RAM usage doesn't exceed 250MB in testing.
-                while len(file_transfers) > 0:
+                while files_to_download:
+                    if self.is_cancelled():
+                        break
                     future_file_transfers = {}
                     for i in range(batch_size):
-                        if len(file_transfers) > 0:
-                            f, filetransfer = file_transfers.pop()
+                        if self.is_cancelled():
+                            break
+                        if files_to_download:
+                            f = files_to_download.pop()
+                            filename = get_content_file_name(f)
+                            try:
+                                dest = paths.get_content_storage_file_path(filename)
+                            except InvalidStorageFilenameError:
+                                # If the destination file name is malformed, just stop now.
+                                overall_progress_update(f["file_size"])
+                                continue
+
+                            # if the file already exists add its size to our overall progress, and skip
+                            if (
+                                os.path.isfile(dest)
+                                and os.path.getsize(dest) == f["file_size"]
+                            ):
+                                overall_progress_update(f["file_size"])
+                                file_checksums_to_annotate.append(f["id"])
+                                transferred_file_size += f["file_size"]
+                                continue
+
+                            # determine where we're downloading/copying from, and create appropriate transfer object
+                            if method == DOWNLOAD_METHOD:
+                                url = paths.get_content_storage_remote_url(
+                                    filename, baseurl=baseurl
+                                )
+                                filetransfer = transfer.FileDownload(
+                                    url,
+                                    dest,
+                                    session=session,
+                                    cancel_check=self.is_cancelled,
+                                )
+                            elif method == COPY_METHOD:
+                                try:
+                                    srcpath = paths.get_content_storage_file_path(
+                                        filename, datafolder=path
+                                    )
+                                except InvalidStorageFilenameError:
+                                    # If the source file name is malformed, just stop now.
+                                    overall_progress_update(f["file_size"])
+                                    continue
+                                filetransfer = transfer.FileCopy(
+                                    srcpath, dest, cancel_check=self.is_cancelled
+                                )
                             future = executor.submit(
                                 self._start_file_transfer, f, filetransfer
                             )
@@ -355,8 +410,16 @@ class Command(AsyncCommand):
                             if status == FILE_SKIPPED:
                                 number_of_skipped_files += 1
                             else:
-                                file_checksums_to_annotate.append(f.id)
-                                transferred_file_size += f.file_size
+                                file_checksums_to_annotate.append(f["id"])
+                                transferred_file_size += f["file_size"]
+                            remaining_bytes_to_transfer -= f["file_size"]
+                            remaining_free_space = get_free_space(
+                                conf.OPTIONS["Paths"]["CONTENT_DIR"]
+                            )
+                            if remaining_free_space <= remaining_bytes_to_transfer:
+                                raise InsufficientStorageSpaceError(
+                                    "Kolibri ran out of storage space while importing content"
+                                )
                         except transfer.TransferCanceled:
                             break
                         except Exception as e:
@@ -368,51 +431,52 @@ class Command(AsyncCommand):
                                 and e.response.status_code == 404
                             ) or (isinstance(e, OSError) and e.errno == 2):
                                 # Continue file import when the current file is not found from the source and is skipped.
-                                overall_progress_update(f.file_size)
+                                overall_progress_update(f["file_size"])
                                 number_of_skipped_files += 1
                                 continue
                             else:
                                 self.exception = e
                                 break
+                    if self.is_cancelled():
+                        for future in future_file_transfers:
+                            future.cancel()
 
-            with db_task_write_lock:
-                annotation.set_content_visibility(
-                    channel_id,
-                    file_checksums_to_annotate,
-                    node_ids=node_ids,
-                    exclude_node_ids=exclude_node_ids,
-                    public=public,
+        annotation.set_content_visibility(
+            channel_id,
+            file_checksums_to_annotate,
+            node_ids=node_ids,
+            exclude_node_ids=exclude_node_ids,
+            public=public,
+        )
+
+        resources_after_transfer = (
+            ContentNode.objects.filter(channel_id=channel_id, available=True)
+            .exclude(kind=content_kinds.TOPIC)
+            .values("content_id")
+            .count()
+        )
+
+        if job:
+            job.extra_metadata["transferred_file_size"] = transferred_file_size
+            job.extra_metadata["transferred_resources"] = (
+                resources_after_transfer - resources_before_transfer
+            )
+            job.save_meta()
+
+        if number_of_skipped_files > 0:
+            logger.warning(
+                "{} files are skipped, because errors occurred during the import.".format(
+                    number_of_skipped_files
                 )
-
-            resources_after_transfer = (
-                ContentNode.objects.filter(channel_id=channel_id, available=True)
-                .exclude(kind=content_kinds.TOPIC)
-                .values("content_id")
-                .distinct()
-                .count()
             )
 
-            if job:
-                job.extra_metadata["transferred_file_size"] = transferred_file_size
-                job.extra_metadata["transferred_resources"] = (
-                    resources_after_transfer - resources_before_transfer
-                )
-                job.save_meta()
+        overall_progress_update(dummy_bytes_for_annotation)
 
-            if number_of_skipped_files > 0:
-                logger.warning(
-                    "{} files are skipped, because errors occurred during the import.".format(
-                        number_of_skipped_files
-                    )
-                )
+        if self.exception:
+            raise self.exception
 
-            overall_progress_update(dummy_bytes_for_annotation)
-
-            if self.exception:
-                raise self.exception
-
-            if self.is_cancelled():
-                self.cancel()
+        if self.is_cancelled():
+            self.cancel()
 
     def _start_file_transfer(self, f, filetransfer):
         """
@@ -432,22 +496,38 @@ class Command(AsyncCommand):
             # the difference so that the overall progress is never incorrect.
             # This could happen, for example for a local transfer if a file
             # has been replaced or corrupted (which we catch below)
-            data_transferred += f.file_size - filetransfer.total_size
+            data_transferred += f["file_size"] - filetransfer.total_size
 
             # If checksum of the destination file is different from the localfile
             # id indicated in the database, it means that the destination file
             # is corrupted, either from origin or during import. Skip importing
             # this file.
-            checksum_correctness = compare_checksums(filetransfer.dest, f.id)
+            try:
+                checksum_correctness = compare_checksums(filetransfer.dest, f["id"])
+            except (IOError, OSError):
+                checksum_correctness = False
             if not checksum_correctness:
                 e = "File {} is corrupted.".format(filetransfer.source)
                 logger.error("An error occurred during content import: {}".format(e))
-                os.remove(filetransfer.dest)
+                try:
+                    os.remove(filetransfer.dest)
+                except OSError:
+                    pass
                 return FILE_SKIPPED, data_transferred
 
         return FILE_TRANSFERRED, data_transferred
 
     def handle_async(self, *args, **options):
+        try:
+            ChannelMetadata.objects.get(id=options["channel_id"])
+        except ValueError:
+            raise CommandError(
+                "{} is not a valid channel_id".format(options["channel_id"])
+            )
+        except ChannelMetadata.DoesNotExist:
+            raise CommandError(
+                "Must import a channel with importchannel before importing content."
+            )
         if options["command"] == "network":
             self.download_content(
                 options["channel_id"],
