@@ -1,8 +1,9 @@
+from collections import OrderedDict
+
 from django.core.management import call_command
 from django.core.management.base import CommandError
 from django.core.urlresolvers import reverse
 from django.test import TestCase
-from mock import ANY
 from mock import call
 from mock import Mock
 from mock import patch
@@ -19,15 +20,23 @@ from kolibri.core.auth.models import FacilityUser
 from kolibri.core.auth.test.test_api import FacilityUserFactory
 from kolibri.core.device.models import DevicePermissions
 from kolibri.core.device.models import DeviceSettings
+from kolibri.core.device.permissions import IsSuperuser
 from kolibri.core.discovery.utils.network.errors import NetworkLocationNotFound
+from kolibri.core.tasks.api import prepare_peer_sync_job
+from kolibri.core.tasks.api import prepare_sync_job
 from kolibri.core.tasks.api import prepare_sync_task
 from kolibri.core.tasks.api import ResourceGoneError
-from kolibri.core.tasks.api import validate_and_prepare_peer_sync_job
-from kolibri.core.tasks.api import validate_prepare_sync_job
+from kolibri.core.tasks.api import validate_and_create_sync_credentials
+from kolibri.core.tasks.api import validate_facility
+from kolibri.core.tasks.api import validate_peer_sync_job
+from kolibri.core.tasks.api import validate_sync_task
+from kolibri.core.tasks.decorators import register_task
 from kolibri.core.tasks.exceptions import JobNotFound
 from kolibri.core.tasks.job import Job
+from kolibri.core.tasks.job import JobRegistry
 from kolibri.core.tasks.job import State
 
+
 DUMMY_PASSWORD = "password"
 
 fake_job_defaults = dict(
@@ -37,7 +46,7 @@ fake_job_defaults = dict(
     traceback="",
     percentage_progress=0,
     cancellable=False,
-    extra_metadata=dict(),
+    extra_metadata={},
     func=lambda: None,
 )
 
@@ -64,24 +73,26 @@ class BaseAPITestCase(APITestCase):
         self.client.login(username=self.superuser.username, password=DUMMY_PASSWORD)
 
 
+@patch("kolibri.core.tasks.api.priority_queue")
 @patch("kolibri.core.tasks.api.queue")
 class TaskAPITestCase(BaseAPITestCase):
-    def test_task_cancel(self, queue_mock):
+    def test_task_cancel(self, queue_mock, priority_queue_mock):
         queue_mock.fetch_job.return_value = fake_job(state=State.CANCELED)
         response = self.client.post(
             reverse("kolibri:core:task-canceltask"), {"task_id": "1"}, format="json"
         )
         self.assertEqual(response.data, {})
 
-    def test_task_cancel_no_task(self, queue_mock):
+    def test_task_cancel_no_task(self, queue_mock, priority_queue_mock):
         queue_mock.cancel.side_effect = JobNotFound()
         response = self.client.post(
             reverse("kolibri:core:task-canceltask"), {"task_id": "1"}, format="json"
         )
         self.assertEqual(response.status_code, 200)
 
-    def test_task_get_no_task(self, queue_mock):
+    def test_task_get_no_task(self, queue_mock, priority_queue_mock):
         queue_mock.fetch_job.side_effect = JobNotFound()
+        priority_queue_mock.fetch_job.side_effect = JobNotFound()
         response = self.client.get(
             reverse("kolibri:core:task-detail", kwargs={"pk": "1"}),
             {"task_id": "1"},
@@ -89,7 +100,409 @@ class TaskAPITestCase(BaseAPITestCase):
         )
         self.assertEqual(response.status_code, 404)
 
+    def test_tasks_clearable_flag(self, queue_mock, priority_queue_mock):
+        queue_mock.jobs = [
+            fake_job(state=state)
+            for state in [
+                # not clearable
+                State.SCHEDULED,
+                State.QUEUED,
+                State.RUNNING,
+                State.CANCELING,
+                # clearable
+                State.FAILED,
+                State.CANCELED,
+                State.COMPLETED,
+            ]
+        ]
+        priority_queue_mock.jobs = []
+        response = self.client.get(reverse("kolibri:core:task-list"))
 
+        def assert_clearable(index, expected):
+            self.assertEqual(response.data[index]["clearable"], expected)
+
+        for i in [0, 1, 2, 3]:
+            assert_clearable(i, False)
+        for i in [4, 5, 6]:
+            assert_clearable(i, True)
+
+    def test_restart_task(self, queue_mock, priority_queue_mock):
+        queue_mock.restart_job.return_value = 1
+        queue_mock.fetch_job.return_value = fake_job(state=State.QUEUED, job_id=1)
+
+        response = self.client.post(
+            reverse("kolibri:core:task-restarttask"), {"task_id": "1"}, format="json"
+        )
+
+        expected_response = {
+            "status": "QUEUED",
+            "exception": "",
+            "traceback": "",
+            "percentage": 0,
+            "id": 1,
+            "cancellable": False,
+            "clearable": False,
+        }
+
+        self.assertDictEqual(response.data, expected_response)
+
+
+@patch("kolibri.core.tasks.api.job_storage")
+@patch("kolibri.core.tasks.job.RegisteredJob.enqueue")
+class CreateTaskAPITestCase(BaseAPITestCase):
+    def tearDown(self):
+        JobRegistry.REGISTERED_JOBS.clear()
+
+    def test_api_validator_task_field_check(self, mock_enqueue, mock_job_storage):
+        # When "task" is absent.
+        response = self.client.post(
+            reverse("kolibri:core:task-list"), {"x": 0, "y": 42}, format="json"
+        )
+        self.assertEqual(response.status_code, 400)
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            [{"x": 0, "y": 42}, {"x": 0, "y": 42}],
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+        # When "task" has a value of incorrect type.
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": 100, "x": 0, "y": 42},
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            [{"task": 100, "x": 0, "y": 42}, {"task": True, "x": 0, "y": 42}],
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_api_validator_unregistered_task(self, mock_enqueue, mock_job_storage):
+        # When "task" is not registered via the decorator.
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {
+                "task": "kolibri.core.tasks.test.test_api.add",
+                "x": 0,
+                "y": 42,
+            },
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            [
+                {
+                    "task": "kolibri.core.tasks.test.test_api.add",
+                    "x": 0,
+                    "y": 42,
+                }
+            ],
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_api_validator_handles_task_permissions(
+        self, mock_enqueue, mock_job_storage
+    ):
+        @register_task(permission_classes=[IsSuperuser])
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        # Let us logout the superuser to send request anonymously.
+        self.client.logout()
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": "kolibri.core.tasks.test.test_api.add", "x": 0, "y": 42},
+            format="json",
+        )
+        self.assertEqual(response.status_code, 403)
+
+    def test_api_errors_on_task_validator_wrong_return_type(
+        self, mock_enqueue, mock_job_storage
+    ):
+        def add_validator(request, req_data):
+            return "kolibri"
+
+        @register_task(validator=add_validator)
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": "kolibri.core.tasks.test.test_api.add", "x": 0, "y": 42},
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_api_reraises_task_validator_exception(
+        self, mock_enqueue, mock_job_storage
+    ):
+        def add_validator(req, req_data):
+            raise TypeError
+
+        @register_task(validator=add_validator)
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        with self.assertRaises(TypeError):
+            self.client.post(
+                reverse("kolibri:core:task-list"),
+                {"task": "kolibri.core.tasks.test.test_api.add", "x": 0, "y": 42},
+                format="json",
+            )
+
+    def test_api_checks_extra_metadata_type(self, mock_enqueue, mock_job_storage):
+        def add_validator(req, req_data):
+            req_data["extra_metadata"] = "string"
+            return req_data
+
+        @register_task(validator=add_validator)
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": "kolibri.core.tasks.test.test_api.add", "x": 0, "y": 42},
+            format="json",
+        )
+        self.assertEqual(response.status_code, 400)
+
+    def test_api_handles_single_task_without_vaidator(
+        self, mock_enqueue, mock_job_storage
+    ):
+        @register_task(permission_classes=[IsSuperuser])
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        mock_enqueue.return_value = "test"
+        mock_job_storage.get_job.return_value = fake_job(
+            state=State.QUEUED, job_id="test"
+        )
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+            format="json",
+        )
+
+        expected_response = {
+            "id": "test",
+            "status": "QUEUED",
+            "exception": "",
+            "traceback": "",
+            "percentage": 0,
+            "cancellable": False,
+            "clearable": False,
+        }
+
+        # Did API return the right stuff?
+        self.assertEqual(response.status_code, 200)
+        self.assertDictEqual(response.data, expected_response)
+
+        # Do we call enqueue the right way i.e. are we passing
+        # the request's data as keyword args to enqueue method?
+        mock_enqueue.assert_called_once_with(**{"kolibri": "fly"})
+
+        # Do we retrieve the task from db to ready the response?
+        mock_job_storage.get_job.assert_called_once_with("test")
+
+    def test_api_handles_bulk_task_without_vaidator(
+        self, mock_enqueue, mock_job_storage
+    ):
+        @register_task(permission_classes=[IsSuperuser])
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        mock_enqueue.return_value = "test"
+        mock_job_storage.get_job.return_value = fake_job(
+            state=State.QUEUED, job_id="test"
+        )
+
+        request_payload = [
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+        ]
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            request_payload,
+            format="json",
+        )
+
+        expected_response = [
+            {
+                "id": "test",
+                "status": "QUEUED",
+                "exception": "",
+                "traceback": "",
+                "percentage": 0,
+                "cancellable": False,
+                "clearable": False,
+            },
+            {
+                "id": "test",
+                "status": "QUEUED",
+                "exception": "",
+                "traceback": "",
+                "percentage": 0,
+                "cancellable": False,
+                "clearable": False,
+            },
+        ]
+
+        # Did API return the right stuff?
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.data, expected_response)
+
+        # Do we call enqueue the right way i.e. are we passing
+        # the request's data as keyword args to enqueue method?
+        self.assertEqual(mock_enqueue.call_count, 2)
+        mock_enqueue.assert_has_calls(
+            [
+                call(**{"kolibri": "fly"}),
+                call(**{"kolibri": "fly"}),
+            ]
+        )
+
+        # Do we retrieve the task from db to ready the response?
+        self.assertEqual(mock_job_storage.get_job.call_count, 2)
+        mock_job_storage.get_job.assert_has_calls([call("test"), call("test")])
+
+    def test_api_handles_single_task_with_vaidator(
+        self, mock_enqueue, mock_job_storage
+    ):
+        def add_validator(req, req_data):
+            # Does validator receives the right arguments?
+            self.assertIsInstance(req, Request)
+            self.assertDictEqual(
+                req_data,
+                {"kolibri": "fly"},
+            )
+            return {"x": 0, "y": 42, "extra_metadata": {"facility": "kolibri HQ"}}
+
+        @register_task(validator=add_validator, permission_classes=[IsSuperuser])
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        mock_enqueue.return_value = "test"
+        mock_job_storage.get_job.return_value = fake_job(
+            state=State.QUEUED, job_id="test", extra_metadata={"facility": "kolibri HQ"}
+        )
+
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+            format="json",
+        )
+
+        expected_response = {
+            "id": "test",
+            "status": "QUEUED",
+            "exception": "",
+            "traceback": "",
+            "percentage": 0,
+            "cancellable": False,
+            "clearable": False,
+            "facility": "kolibri HQ",
+        }
+
+        # Did API return the right stuff?
+        self.assertEqual(response.status_code, 200)
+        self.assertDictEqual(response.data, expected_response)
+
+        # Do we call enqueue the right way i.e. are we passing
+        # the return value of validator as keyword args to enqueue method?
+        mock_enqueue.assert_called_once_with(
+            **{"x": 0, "y": 42, "extra_metadata": {"facility": "kolibri HQ"}}
+        )
+
+        # Do we retrieve the task from db to ready the response?
+        mock_job_storage.get_job.assert_called_once_with("test")
+
+    def test_api_handles_bulk_task_with_validator(self, mock_enqueue, mock_job_storage):
+        def add_validator(req, req_data):
+            # Does validator receives the right arguments?
+            self.assertIsInstance(req, Request)
+            self.assertDictEqual(
+                req_data,
+                {"kolibri": "fly"},
+            )
+            return {"x": 0, "y": 42, "extra_metadata": {"facility": "kolibri HQ"}}
+
+        @register_task(validator=add_validator, permission_classes=[IsSuperuser])
+        def add(**kwargs):
+            return kwargs["x"] + kwargs["y"]
+
+        mock_enqueue.return_value = "test"
+        mock_job_storage.get_job.return_value = fake_job(
+            state=State.QUEUED, job_id="test", extra_metadata={"facility": "kolibri HQ"}
+        )
+
+        request_payload = [
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+            {"task": "kolibri.core.tasks.test.test_api.add", "kolibri": "fly"},
+        ]
+        response = self.client.post(
+            reverse("kolibri:core:task-list"),
+            request_payload,
+            format="json",
+        )
+
+        expected_response = [
+            {
+                "id": "test",
+                "status": "QUEUED",
+                "exception": "",
+                "traceback": "",
+                "percentage": 0,
+                "cancellable": False,
+                "clearable": False,
+                "facility": "kolibri HQ",
+            },
+            {
+                "id": "test",
+                "status": "QUEUED",
+                "exception": "",
+                "traceback": "",
+                "percentage": 0,
+                "cancellable": False,
+                "clearable": False,
+                "facility": "kolibri HQ",
+            },
+        ]
+
+        # Did API return the right stuff?
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.data, expected_response)
+
+        # Do we call enqueue the right way i.e. are we passing
+        # the return value of validator as keyword args to enqueue method?
+        self.assertEqual(mock_enqueue.call_count, 2)
+        mock_enqueue.assert_has_calls(
+            [
+                call(**{"x": 0, "y": 42, "extra_metadata": {"facility": "kolibri HQ"}}),
+                call(**{"x": 0, "y": 42, "extra_metadata": {"facility": "kolibri HQ"}}),
+            ]
+        )
+
+        # Do we retrieve the task from db to ready the response?
+        self.assertEqual(mock_job_storage.get_job.call_count, 2)
+        mock_job_storage.get_job.assert_has_calls([call("test"), call("test")])
+
+
+@patch("kolibri.core.tasks.api.priority_queue")
+@patch("kolibri.core.tasks.api.queue")
 class TaskAPIPermissionsTestCase(APITestCase):
     def setUp(self):
         DeviceSettings.objects.create(is_provisioned=True)
@@ -98,7 +511,7 @@ class TaskAPIPermissionsTestCase(APITestCase):
         self.facility.add_admin(admin)
         self.client.login(username=admin.username, password=DUMMY_PASSWORD)
 
-    def test_exportlogs_permissions(self):
+    def test_exportlogs_permissions(self, queue_mock, priority_queue_mock):
         with patch("kolibri.core.tasks.api._job_to_response", return_value={}):
             response = self.client.post(
                 reverse("kolibri:core:task-startexportlogcsv"),
@@ -107,13 +520,13 @@ class TaskAPIPermissionsTestCase(APITestCase):
             )
         self.assertEqual(response.status_code, 200)
 
-    def test_list_permissions(self):
+    def test_list_permissions(self, queue_mock, priority_queue_mock):
         with patch("kolibri.core.tasks.api._job_to_response", return_value={}):
             response = self.client.get(reverse("kolibri:core:task-list"), format="json")
         self.assertEqual(response.status_code, 200)
 
 
-@patch("kolibri.core.tasks.api.facility_queue", spec=True)
+@patch("kolibri.core.tasks.api.facility_queue")
 class FacilityTaskAPITestCase(BaseAPITestCase):
     def assertJobResponse(self, job_data, response):
         id = job_data.get("job_id", fake_job_defaults.get("job_id"))
@@ -270,12 +683,14 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
             any_order=True,
         )
 
-    @patch("kolibri.core.tasks.api.validate_and_prepare_peer_sync_job")
-    @patch("kolibri.core.tasks.api.get_client_and_server_certs")
+    @patch("kolibri.core.tasks.api.validate_peer_sync_job")
+    @patch("kolibri.core.tasks.api.prepare_peer_sync_job")
+    @patch("kolibri.core.tasks.api.validate_and_create_sync_credentials")
     def test_startpeerfacilityimport(
         self,
-        get_client_and_server_certs,
-        validate_and_prepare_peer_sync_job,
+        validate_and_create_sync_credentials,
+        prepare_peer_sync_job,
+        validate_peer_sync_job,
         facility_queue,
     ):
         user = self.superuser
@@ -293,6 +708,10 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
             device_id="",
             baseurl="https://some.server.test/extra/stuff",
         )
+        request_data = OrderedDict(baseurl="https://some.server.test/")
+        request_data["facility"] = self.facility.id
+        request_data["username"] = ""
+        request_data["password"] = ""
         prepared_data = dict(
             baseurl="https://some.server.test/",
             facility=self.facility.id,
@@ -303,7 +722,8 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
             track_progress=True,
             cancellable=False,
         )
-        validate_and_prepare_peer_sync_job.return_value = prepared_data.copy()
+        validate_peer_sync_job.return_value = request_data.copy()
+        prepare_peer_sync_job.return_value = prepared_data.copy()
 
         facility_queue.enqueue.return_value = 123
         fake_job_data = dict(
@@ -327,15 +747,28 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
         )
         self.assertEqual(response.status_code, 200)
         self.assertJobResponse(fake_job_data, response)
-
-        validate_and_prepare_peer_sync_job.assert_has_calls(
-            [call(ANY, no_push=True, no_provision=True, extra_metadata=extra_metadata)]
+        prepare_peer_sync_job.assert_has_calls(
+            [
+                call(
+                    "baseurl",
+                    "facility",
+                    no_push=True,
+                    no_provision=True,
+                    extra_metadata=extra_metadata,
+                )
+            ]
         )
         facility_queue.enqueue.assert_called_with(call_command, "sync", **prepared_data)
 
-    @patch("kolibri.core.tasks.api.validate_and_prepare_peer_sync_job")
+    @patch("kolibri.core.tasks.api.prepare_peer_sync_job")
+    @patch("kolibri.core.tasks.api.validate_peer_sync_job")
+    @patch("kolibri.core.tasks.api.validate_and_create_sync_credentials")
     def test_startpeerfacilitysync(
-        self, validate_and_prepare_peer_sync_job, facility_queue
+        self,
+        validate_and_create_sync_credentials,
+        validate_peer_sync_job,
+        prepare_peer_sync_job,
+        facility_queue,
     ):
         user = self.superuser
 
@@ -352,7 +785,11 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
             device_id="",
             baseurl="https://some.server.test/extra/stuff",
         )
-        prepared_data = dict(
+        request_data = OrderedDict(baseurl="https://some.server.test/")
+        request_data["facility"] = self.facility.id
+        request_data["username"] = ""
+        request_data["password"] = ""
+        prepared_data = OrderedDict(
             baseurl="https://some.server.test/",
             facility=self.facility.id,
             chunk_size=200,
@@ -361,7 +798,8 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
             track_progress=True,
             cancellable=False,
         )
-        validate_and_prepare_peer_sync_job.return_value = prepared_data.copy()
+        validate_peer_sync_job.return_value = request_data.copy()
+        prepare_peer_sync_job.return_value = prepared_data.copy()
 
         facility_queue.enqueue.return_value = 123
         fake_job_data = dict(
@@ -385,9 +823,8 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
         )
         self.assertEqual(response.status_code, 200)
         self.assertJobResponse(fake_job_data, response)
-
-        validate_and_prepare_peer_sync_job.assert_has_calls(
-            [call(ANY, extra_metadata=extra_metadata)]
+        prepare_peer_sync_job.assert_has_calls(
+            [call("baseurl", "facility", extra_metadata=extra_metadata)]
         )
         facility_queue.enqueue.assert_called_with(call_command, "sync", **prepared_data)
 
@@ -475,7 +912,7 @@ class FacilityTaskAPITestCase(BaseAPITestCase):
 
 
 class FacilityTaskHelperTestCase(TestCase):
-    def test_prepare_sync_task(self):
+    def test_validate_sync_task(self):
         user = Mock(spec=FacilityUser, pk=456, username="abc")
         req = Mock(spec=Request, data=dict(facility=123), user=user)
 
@@ -488,12 +925,10 @@ class FacilityTaskHelperTestCase(TestCase):
             bytes_received=0,
             other_kwarg="is test",
         )
-        actual = prepare_sync_task(req, other_kwarg="is test")
+        actual = prepare_sync_task(*validate_sync_task(req), other_kwarg="is test")
         self.assertEqual(expected, actual)
 
-    def test_validate_prepare_sync_job(self):
-        req = Mock(spec=Request, data=dict(facility=123))
-
+    def test__prepare_sync_job(self):
         expected = dict(
             facility=123,
             chunk_size=200,
@@ -502,28 +937,28 @@ class FacilityTaskHelperTestCase(TestCase):
             cancellable=False,
             extra_metadata=dict(type="test"),
         )
-        actual = validate_prepare_sync_job(req, extra_metadata=dict(type="test"))
+        actual = prepare_sync_job(facility=123, extra_metadata=dict(type="test"))
         self.assertEqual(expected, actual)
 
-    def test_validate_prepare_sync_job__parse_error(self):
-        req = Mock(spec="rest_framework.requests.Request", data=dict())
+    def test_validate_facility__parse_error(self):
+        req = Mock(spec="rest_framework.requests.Request", data={})
 
         with self.assertRaises(ParseError):
-            validate_prepare_sync_job(req, extra_metadata=dict(type="test"))
+            validate_facility(req)
 
-    def test_validate_prepare_sync_job__parse_error__empty(self):
+    def test_validate_facility__parse_error__empty(self):
         req = Mock(spec="rest_framework.requests.Request", data=dict(facility=""))
 
         with self.assertRaises(ParseError):
-            validate_prepare_sync_job(req, extra_metadata=dict(type="test"))
+            validate_facility(req)
 
     @patch("kolibri.core.tasks.api.MorangoProfileController")
     @patch("kolibri.core.tasks.api.NetworkClient")
     @patch("kolibri.core.tasks.api.get_client_and_server_certs")
-    @patch("kolibri.core.tasks.api.get_dataset_id")
-    def test_validate_and_prepare_peer_sync_job(
+    @patch("kolibri.core.tasks.api.get_facility_dataset_id")
+    def test_validate_peer_sync_job(
         self,
-        get_dataset_id,
+        get_facility_dataset_id,
         get_client_and_server_certs,
         NetworkClient,
         MorangoProfileController,
@@ -546,7 +981,7 @@ class FacilityTaskHelperTestCase(TestCase):
         controller = MorangoProfileController.return_value
         controller.create_network_connection.return_value = network_connection
 
-        get_dataset_id.return_value = dataset_id
+        get_facility_dataset_id.return_value = (123, dataset_id)
         get_client_and_server_certs.return_value = None
 
         expected = dict(
@@ -558,8 +993,10 @@ class FacilityTaskHelperTestCase(TestCase):
             cancellable=False,
             extra_metadata=dict(type="test"),
         )
-        actual = validate_and_prepare_peer_sync_job(
-            req, extra_metadata=dict(type="test")
+        req_params = validate_peer_sync_job(req)
+        validate_and_create_sync_credentials(*req_params)
+        actual = prepare_peer_sync_job(
+            *req_params[:2], extra_metadata=dict(type="test")
         )
         self.assertEqual(expected, actual)
 
@@ -568,30 +1005,36 @@ class FacilityTaskHelperTestCase(TestCase):
             "https://some.server.test/"
         )
 
-        get_dataset_id.assert_called_with(
+        get_facility_dataset_id.assert_called_with(
             "https://some.server.test/", identifier=123, noninteractive=True
         )
 
         get_client_and_server_certs.assert_called_with(
-            "tester", "mypassword", dataset_id, network_connection, noninteractive=True
+            "tester",
+            "mypassword",
+            dataset_id,
+            network_connection,
+            user_id=None,
+            facility_id=123,
+            noninteractive=True,
         )
 
-    def test_validate_and_prepare_peer_sync_job__no_baseurl(self):
+    def test_validate_peer_sync_job__no_baseurl(self):
         req = Mock(spec=Request, data=dict(facility=123))
 
         with self.assertRaises(ParseError, msg="Missing `baseurl` parameter"):
-            validate_and_prepare_peer_sync_job(req)
+            validate_peer_sync_job(req)
 
-    def test_validate_and_prepare_peer_sync_job__bad_url(self):
+    def test_validate_peer_sync_job__bad_url(self):
         req = Mock(
             spec=Request, data=dict(facility=123, baseurl="/com.bad.url.www//:sptth")
         )
 
         with self.assertRaises(ParseError, msg="Invalid URL"):
-            validate_and_prepare_peer_sync_job(req)
+            validate_peer_sync_job(req)
 
     @patch("kolibri.core.tasks.api.NetworkClient")
-    def test_validate_and_prepare_peer_sync_job__cannot_connect(self, NetworkClient):
+    def test_validate_peer_sync_job__cannot_connect(self, NetworkClient):
         req = Mock(
             spec=Request, data=dict(facility=123, baseurl="https://www.notfound.never")
         )
@@ -599,13 +1042,13 @@ class FacilityTaskHelperTestCase(TestCase):
         NetworkClient.side_effect = NetworkLocationNotFound()
 
         with self.assertRaises(ResourceGoneError):
-            validate_and_prepare_peer_sync_job(req)
+            validate_peer_sync_job(req)
 
     @patch("kolibri.core.tasks.api.MorangoProfileController")
     @patch("kolibri.core.tasks.api.NetworkClient")
-    @patch("kolibri.core.tasks.api.get_dataset_id")
-    def test_validate_and_prepare_peer_sync_job__unknown_facility(
-        self, get_dataset_id, NetworkClient, MorangoProfileController
+    @patch("kolibri.core.tasks.api.get_facility_dataset_id")
+    def test_validate_and_create_sync_credentials__unknown_facility(
+        self, get_facility_dataset_id, NetworkClient, MorangoProfileController
     ):
         req = Mock(
             spec=Request,
@@ -624,18 +1067,18 @@ class FacilityTaskHelperTestCase(TestCase):
         controller = MorangoProfileController.return_value
         controller.create_network_connection.return_value = network_connection
 
-        get_dataset_id.side_effect = CommandError()
+        get_facility_dataset_id.side_effect = CommandError()
 
         with self.assertRaises(AuthenticationFailed):
-            validate_and_prepare_peer_sync_job(req, extra_metadata=dict(type="test"))
+            validate_and_create_sync_credentials(*validate_peer_sync_job(req))
 
     @patch("kolibri.core.tasks.api.MorangoProfileController")
     @patch("kolibri.core.tasks.api.NetworkClient")
     @patch("kolibri.core.tasks.api.get_client_and_server_certs")
-    @patch("kolibri.core.tasks.api.get_dataset_id")
-    def test_validate_and_prepare_peer_sync_job__not_authenticated(
+    @patch("kolibri.core.tasks.api.get_facility_dataset_id")
+    def test_validate_and_create_sync_credentials__not_authenticated(
         self,
-        get_dataset_id,
+        get_facility_dataset_id,
         get_client_and_server_certs,
         NetworkClient,
         MorangoProfileController,
@@ -652,19 +1095,19 @@ class FacilityTaskHelperTestCase(TestCase):
         controller = MorangoProfileController.return_value
         controller.create_network_connection.return_value = network_connection
 
-        get_dataset_id.return_value = 456
+        get_facility_dataset_id.return_value = (123, 456)
         get_client_and_server_certs.side_effect = CommandError()
 
         with self.assertRaises(PermissionDenied):
-            validate_and_prepare_peer_sync_job(req, extra_metadata=dict(type="test"))
+            validate_and_create_sync_credentials(*validate_peer_sync_job(req))
 
     @patch("kolibri.core.tasks.api.MorangoProfileController")
     @patch("kolibri.core.tasks.api.NetworkClient")
     @patch("kolibri.core.tasks.api.get_client_and_server_certs")
-    @patch("kolibri.core.tasks.api.get_dataset_id")
-    def test_validate_and_prepare_peer_sync_job__authentication_failed(
+    @patch("kolibri.core.tasks.api.get_facility_dataset_id")
+    def test_validate_and_create_sync_credentials__authentication_failed(
         self,
-        get_dataset_id,
+        get_facility_dataset_id,
         get_client_and_server_certs,
         NetworkClient,
         MorangoProfileController,
@@ -686,8 +1129,8 @@ class FacilityTaskHelperTestCase(TestCase):
         controller = MorangoProfileController.return_value
         controller.create_network_connection.return_value = network_connection
 
-        get_dataset_id.return_value = 456
+        get_facility_dataset_id.return_value = (123, 456)
         get_client_and_server_certs.side_effect = CommandError()
 
         with self.assertRaises(AuthenticationFailed):
-            validate_and_prepare_peer_sync_job(req, extra_metadata=dict(type="test"))
+            validate_and_create_sync_credentials(*validate_peer_sync_job(req))
