@@ -2,11 +2,10 @@ import datetime
 
 from django.db import connections
 from django.db.models import Count
-from django.db.models import F
-from django.db.models import Q
 from django.db.models import Sum
 from django.db.utils import DatabaseError
 from django.db.utils import OperationalError
+from django.http import Http404
 from rest_framework import permissions
 from rest_framework import viewsets
 from rest_framework.response import Response
@@ -15,18 +14,17 @@ from .serializers import LessonReportSerializer
 from kolibri.core.api import ValuesViewset
 from kolibri.core.auth.constants import collection_kinds
 from kolibri.core.auth.constants import role_kinds
-from kolibri.core.auth.filters import HierarchyRelationsFilter
 from kolibri.core.auth.models import Collection
 from kolibri.core.auth.models import FacilityUser
 from kolibri.core.decorators import query_params_required
 from kolibri.core.exams.models import Exam
 from kolibri.core.lessons.models import Lesson
 from kolibri.core.logger.models import AttemptLog
-from kolibri.core.logger.models import ExamAttemptLog
-from kolibri.core.logger.models import ExamLog
+from kolibri.core.logger.models import MasteryLog
 from kolibri.core.notifications.models import LearnerProgressNotification
 from kolibri.core.notifications.models import NotificationsLog
 from kolibri.core.sqlite.utils import repair_sqlite_db
+from kolibri.deployment.default.sqlite_db_names import NOTIFICATIONS
 
 collection_kind_choices = tuple(
     [choice[0] for choice in collection_kinds.choices] + ["user"]
@@ -101,10 +99,7 @@ class ClassroomNotificationsViewset(ValuesViewset):
         "notification_event",
     )
 
-    field_map = {
-        "object": "notification_object",
-        "event": "notification_event",
-    }
+    field_map = {"object": "notification_object", "event": "notification_event"}
 
     def check_after(self):
         """
@@ -205,7 +200,7 @@ class ClassroomNotificationsViewset(ValuesViewset):
             except (LearnerProgressNotification.DoesNotExist):
                 return LearnerProgressNotification.objects.none()
             except DatabaseError:
-                repair_sqlite_db(connections["notifications_db"])
+                repair_sqlite_db(connections[NOTIFICATIONS])
                 return LearnerProgressNotification.objects.none()
 
         limit = self.check_limit()
@@ -229,9 +224,9 @@ class ClassroomNotificationsViewset(ValuesViewset):
         are requesting notifications in the last five minutes
         """
         try:
-            queryset = self.filter_queryset(self.prefetch_queryset(self.get_queryset()))
+            queryset = self.filter_queryset(self.get_queryset())
         except (OperationalError, DatabaseError):
-            repair_sqlite_db(connections["notifications_db"])
+            repair_sqlite_db(connections[NOTIFICATIONS])
 
         # L
         logging_interval = datetime.datetime.now() - datetime.timedelta(minutes=5)
@@ -244,7 +239,7 @@ class ClassroomNotificationsViewset(ValuesViewset):
             )
         except (OperationalError, DatabaseError):
             logged_notifications = 0
-            repair_sqlite_db(connections["notifications_db"])
+            repair_sqlite_db(connections[NOTIFICATIONS])
         # if there are more than 10 notifications we limit the answer to 10
         if logged_notifications < 10:
             notification_info = NotificationsLog()
@@ -330,25 +325,23 @@ class ExerciseDifficultQuestionsViewset(BaseExerciseDifficultQuestionsViewset):
             else:
                 # Only filter by all the collections in the lesson if we are not also
                 # filtering by a specific group. Otherwise the group should be sufficient.
-                base_queryset = queryset
-                # Set starting queryset to null, then OR.
-                queryset = AttemptLog.objects.none()
-                for collection_id in collection_ids:
-                    queryset |= HierarchyRelationsFilter(
-                        base_queryset
-                    ).filter_by_hierarchy(
-                        ancestor_collection=collection_id, target_user=F("user")
-                    )
-                queryset = queryset.distinct()
+                queryset = queryset.filter(
+                    user__memberships__collection_id__in=collection_ids
+                )
         if group_id is not None:
             collection_id = group_id or classroom_id
-            queryset = HierarchyRelationsFilter(queryset).filter_by_hierarchy(
-                ancestor_collection=collection_id, target_user=F("user")
-            )
+            queryset = queryset.filter(user__memberships__collection_id=collection_id)
 
         data = (
-            queryset.values("item")
-            .annotate(total=Count("correct"))
+            # Use a subquery to prevent duplication of attempt logs due to the double JOIN
+            # if we have multiple collections that a user is a member of
+            AttemptLog.objects.filter(id__in=queryset.values_list("id", flat=True))
+            .values("item")
+            .annotate(
+                total=Count(
+                    "correct",
+                )
+            )
             .annotate(correct=Sum("correct"))
         )
         return Response(data)
@@ -383,31 +376,33 @@ class QuizDifficultQuestionsViewset(viewsets.ViewSet):
         # Only return logs when the learner has submitted the Quiz OR
         # the coach has deactivated the Quiz. Do not return logs when Quiz is still
         # in-progress.
-        queryset = ExamAttemptLog.objects.filter(
-            Q(examlog__closed=True) | Q(examlog__exam__active=False), examlog__exam=pk
-        )
+        try:
+            quiz = Exam.objects.all().values("active", "collection_id").get(pk=pk)
+        except Exam.DoesNotExist:
+            raise Http404
+        quiz_active = quiz["active"]
+        queryset = AttemptLog.objects.filter(sessionlog__content_id=pk)
+        if quiz_active:
+            queryset = queryset.filter(masterylog__complete=True)
         if group_id is not None:
-            queryset = HierarchyRelationsFilter(queryset).filter_by_hierarchy(
-                ancestor_collection=group_id, target_user=F("user")
-            )
+            queryset = queryset.filter(
+                user__memberships__collection_id=group_id
+            ).distinct()
             collection_id = group_id
         else:
-            collection_id = Exam.objects.get(pk=pk).collection_id
-        data = queryset.values("item", "content_id").annotate(correct=Sum("correct"))
+            collection_id = quiz["collection_id"]
+        data = queryset.values("item").annotate(correct=Sum("correct"))
 
         # Instead of inferring the totals from the number of logs, use the total
         # number of people who submitted (if quiz is active) or started the exam
         # (if quiz is inactive) as our guide, as people who started the exam
         # but did not attempt the question are still important.
+        total_queryset = MasteryLog.objects.filter(summarylog__content_id=pk)
+        if quiz_active:
+            total_queryset = total_queryset.filter(complete=True)
         total = (
-            HierarchyRelationsFilter(
-                ExamLog.objects.filter(
-                    Q(closed=True) | Q(exam__active=False), exam_id=pk
-                )
-            )
-            .filter_by_hierarchy(
-                ancestor_collection=collection_id, target_user=F("user")
-            )
+            total_queryset.filter(user__memberships__collection_id=collection_id)
+            .distinct()
             .count()
         )
         for datum in data:
